import "actor.dart";
import "actor_targeted_constraint.dart";
import "transform_space.dart";
import "binary_reader.dart";

abstract class ActorAxisConstraint extends ActorTargetedConstraint
{
    bool _copyX = false;
    bool _copyY = false;
    bool _enableMinX = false;
    bool _enableMaxX = false;
    bool _enableMinY = false;
    bool _enableMaxY = false;
    bool _offset = false;
    
    double _scaleX = 1.0;
    double _scaleY = 1.0;
    double _minX = 0.0;
    double _maxX = 0.0;
    double _minY = 0.0;
    double _maxY = 0.0;

    int _sourceSpace = TransformSpace.World;
    int _destSpace = TransformSpace.World;
    int _minMaxSpace = TransformSpace.World;

    ActorAxisConstraint() : super();

    static ActorAxisConstraint read(Actor actor, BinaryReader reader, ActorAxisConstraint component)
    {
        ActorTargetedConstraint.read(actor, reader, component);
        component._copyX = reader.readUint8() == 1;
        if(component._copyX)
        {
            component._scaleX = reader.readFloat32();
        }

        component._enableMinX = reader.readUint8() == 1;
        if(component._enableMinX)
        {
            component._minX = reader.readFloat32();
        }

        component._enableMaxX = reader.readUint8() == 1;
        if(component._enableMaxX)
        {
            component._maxX = reader.readFloat32();
        }

        component._copyY = reader.readUint8() == 1;
        if(component._copyY)
        {
            component._scaleY = reader.readFloat32();
        }

        component._enableMinY = reader.readUint8() == 1;
        if(component._enableMinY)
        {
            component._minY = reader.readFloat32();
        }

        component._enableMaxY = reader.readUint8() == 1;
        if(component._enableMaxY)
        {
            component._maxY = reader.readFloat32();
        }

        component._offset = reader.readUint8() == 1;
        component._sourceSpace = reader.readUint8();
        component._destSpace = reader.readUint8();
        component._minMaxSpace = reader.readUint8();

        return component;
    }

	void copyAxisConstraint(ActorAxisConstraint node, Actor resetActor)
    {
        copyTargetedConstraint(node, resetActor);

        _copyX = node._copyX;
        _copyY = node._copyY;
        _scaleX = node._scaleX;
        _scaleY = node._scaleY;
        _enableMinX = node._enableMinX;
        _minX = node._minX;
        _enableMaxX = node._enableMaxX;
        _maxX = node._maxX;
        _enableMinY = node._enableMinY;
		_minY = node._minY;
		_enableMaxY = node._enableMaxY;
		_maxY = node._maxY;
		_offset = node._offset;
		_sourceSpace = node._sourceSpace;
		_destSpace = node._destSpace;
		_minMaxSpace = node._minMaxSpace;
    }

    @override
    onDirty(int dirt)
    {
        markDirty();
    }

    get copyX => _copyX;
    get copyY => _copyY;
    get destSpace => _destSpace;
    get enableMaxX => _enableMaxX;
    get enableMaxY => _enableMaxY;
    get enableMinX => _enableMinX;
    get enableMinY => _enableMinY;
    get maxX => _maxX;
    get maxY => _maxY;
    get minMaxSpace => _minMaxSpace;
    get minX => _minX;
    get minY => _minY;
    get offset => _offset;
    get scaleX => _scaleX;
    get scaleY => _scaleY;
    get sourceSpace => _sourceSpace;
}